@@ -1100,7 +1100,7 @@ st_optionneg(g)
 	{
 		/*
 		**  Older MTAs do not support some protocol steps.
-		**  As this protocol is a bit "wierd" (it asks for steps
+		**  As this protocol is a bit "weird" (it asks for steps
 		**  NOT to be taken/sent) we have to check whether we
 		**  should turn off those "negative" requests.
 		**  Currently these are only SMFIP_NODATA and SMFIP_NOUNKNOWN.
@@ -1186,7 +1186,7 @@ st_connectinfo(g)
 	s = g->a_buf;
 	i = 0;
 	l = g->a_len;
-	while (s[i] != '\0' && i <= l)
+	while (i <= l && s[i] != '\0')  
 		++i;
 	if (i + 1 >= l)
 		return _SMFIS_ABORT;
