@@ -8,7 +8,7 @@
 */
 
 #include <sm/gen.h>
-SM_RCSID("@(#)$Id: smdb2.c,v 8.83 2013/11/22 20:51:49 ca Exp $")
+SM_RCSID("@(#)$Id: smdb2.c,v 8.83 2013-11-22 20:51:49 ca Exp $")
 
 #include <fcntl.h>
 #include <stdlib.h>
@@ -41,7 +41,7 @@ typedef struct smdb_db2_database SMDB_DB2_DATABASE;
 **
 */
 
-DBTYPE
+static DBTYPE
 smdb_type_to_db2_type(type)
 	SMDB_DBTYPE type;
 {
@@ -68,7 +68,7 @@ smdb_type_to_db2_type(type)
 **
 */
 
-int
+static int
 db2_error_to_smdb(error)
 	int error;
 {
@@ -153,7 +153,7 @@ db2_error_to_smdb(error)
 **
 */
 
-unsigned int
+static unsigned int
 smdb_put_flags_to_db2_flags(flags)
 	SMDB_FLAG flags;
 {
@@ -181,7 +181,7 @@ smdb_put_flags_to_db2_flags(flags)
 **
 */
 
-int
+static int
 smdb_cursor_get_flags_to_db2(flags)
 	SMDB_FLAG flags;
 {
@@ -209,7 +209,7 @@ smdb_cursor_get_flags_to_db2(flags)
 **  interface laid out in smdb.h.
 */
 
-SMDB_DB2_DATABASE *
+static SMDB_DB2_DATABASE *
 smdb2_malloc_database()
 {
 	SMDB_DB2_DATABASE *db2;
@@ -221,7 +221,7 @@ smdb2_malloc_database()
 	return db2;
 }
 
-int
+static int
 smdb2_close(database)
 	SMDB_DATABASE *database;
 {
@@ -239,7 +239,7 @@ smdb2_close(database)
 	return result;
 }
 
-int
+static int
 smdb2_del(database, key, flags)
 	SMDB_DATABASE *database;
 	SMDB_DBENT *key;
@@ -254,7 +254,7 @@ smdb2_del(database, key, flags)
 	return db2_error_to_smdb(db->del(db, NULL, &dbkey, flags));
 }
 
-int
+static int
 smdb2_fd(database, fd)
 	SMDB_DATABASE *database;
 	int *fd;
@@ -264,7 +264,7 @@ smdb2_fd(database, fd)
 	return db2_error_to_smdb(db->fd(db, fd));
 }
 
-int
+static int
 smdb2_lockfd(database)
 	SMDB_DATABASE *database;
 {
@@ -273,7 +273,7 @@ smdb2_lockfd(database)
 	return db2->smdb2_lock_fd;
 }
 
-int
+static int
 smdb2_get(database, key, data, flags)
 	SMDB_DATABASE *database;
 	SMDB_DBENT *key;
@@ -295,7 +295,7 @@ smdb2_get(database, key, data, flags)
 	return db2_error_to_smdb(result);
 }
 
-int
+static int
 smdb2_put(database, key, data, flags)
 	SMDB_DATABASE *database;
 	SMDB_DBENT *key;
@@ -317,7 +317,7 @@ smdb2_put(database, key, data, flags)
 }
 
 
-int
+static int
 smdb2_set_owner(database, uid, gid)
 	SMDB_DATABASE *database;
 	uid_t uid;
@@ -340,7 +340,7 @@ smdb2_set_owner(database, uid, gid)
 	return SMDBE_OK;
 }
 
-int
+static int
 smdb2_sync(database, flags)
 	SMDB_DATABASE *database;
 	unsigned int flags;
@@ -350,7 +350,7 @@ smdb2_sync(database, flags)
 	return db2_error_to_smdb(db->sync(db, flags));
 }
 
-int
+static int
 smdb2_cursor_close(cursor)
 	SMDB_CURSOR *cursor;
 {
@@ -362,7 +362,7 @@ smdb2_cursor_close(cursor)
 	return ret;
 }
 
-int
+static int
 smdb2_cursor_del(cursor, flags)
 	SMDB_CURSOR *cursor;
 	SMDB_FLAG flags;
@@ -372,7 +372,7 @@ smdb2_cursor_del(cursor, flags)
 	return db2_error_to_smdb(dbc->c_del(dbc, 0));
 }
 
-int
+static int
 smdb2_cursor_get(cursor, key, value, flags)
 	SMDB_CURSOR *cursor;
 	SMDB_DBENT *key;
@@ -398,7 +398,7 @@ smdb2_cursor_get(cursor, key, value, flags)
 	return db2_error_to_smdb(result);
 }
 
-int
+static int
 smdb2_cursor_put(cursor, key, value, flags)
 	SMDB_CURSOR *cursor;
 	SMDB_DBENT *key;
@@ -418,7 +418,7 @@ smdb2_cursor_put(cursor, key, value, flags)
 	return db2_error_to_smdb(dbc->c_put(dbc, &dbkey, &dbdata, 0));
 }
 
-int
+static int
 smdb2_cursor(database, cursor, flags)
 	SMDB_DATABASE *database;
 	SMDB_CURSOR **cursor;
