@@ -20,11 +20,15 @@ SM_RCSID("@(#)$Id: mci.c,v 8.225 2013-11-22 20:51:56 ca Exp $")
 #endif /* NETINET || NETINET6 */
 
 #include <dirent.h>
+#if STARTTLS
+# include <tls.h>
+#endif
 
 static int	mci_generate_persistent_path __P((const char *, char *,
 						  int, bool));
 static bool	mci_load_persistent __P((MCI *));
 static void	mci_uncache __P((MCI **, bool));
+static void	mci_clear __P((MCI *));
 static int	mci_lock_host_statfile __P((MCI *));
 static int	mci_read_persistent __P((SM_FILE_T *, MCI *));
 
@@ -253,6 +257,7 @@ mci_uncache(mcislot, doquit)
 	SM_FREE_CLR(mci->mci_status);
 	SM_FREE_CLR(mci->mci_rstatus);
 	SM_FREE_CLR(mci->mci_heloname);
+	mci_clear(mci);
 	if (mci->mci_rpool != NULL)
 	{
 		sm_rpool_free(mci->mci_rpool);
@@ -314,6 +319,41 @@ mci_clr_extensions(mci)
 #endif /* SASL */
 }
 
+/*
+**  MCI_CLEAR -- clear mci
+**
+**	Parameters:
+**		mci -- the connection to clear.
+**
+**	Returns:
+**		none.
+*/
+
+static void
+mci_clear(mci)
+	MCI *mci;
+{
+	if (mci == NULL)
+		return;
+
+	mci->mci_maxsize = 0;
+	mci->mci_min_by = 0;
+	mci->mci_deliveries = 0;
+#if SASL
+	if (bitset(MCIF_AUTHACT, mci->mci_flags))
+		sasl_dispose(&mci->mci_conn);
+#endif
+#if STARTTLS
+	if (bitset(MCIF_TLSACT, mci->mci_flags) && mci->mci_ssl != NULL)
+		SM_SSL_FREE(mci->mci_ssl);
+#endif
+
+	/* which flags to preserve? */
+	mci->mci_flags &= MCIF_CACHED;
+	mactabclear(&mci->mci_macro);
+}
+
+
 /*
 **  MCI_GET -- get information about a particular host
 **
@@ -419,6 +459,7 @@ mci_get(host, m)
 			mci->mci_errno = 0;
 			mci->mci_exitstat = EX_OK;
 		}
+		mci_clear(mci);
 	}
 
 	return mci;
@@ -580,7 +621,6 @@ struct mcifbits
 };
 static struct mcifbits	MciFlags[] =
 {
-	{ MCIF_VALID,		"VALID"		},
 	{ MCIF_CACHED,		"CACHED"	},
 	{ MCIF_ESMTP,		"ESMTP"		},
 	{ MCIF_EXPN,		"EXPN"		},
@@ -598,11 +638,14 @@ static struct mcifbits	MciFlags[] =
 	{ MCIF_AUTHACT,		"AUTHACT"	},
 	{ MCIF_ENHSTAT,		"ENHSTAT"	},
 	{ MCIF_PIPELINED,	"PIPELINED"	},
+	{ MCIF_VERB,		"VERB"	},
 #if STARTTLS
 	{ MCIF_TLS,		"TLS"		},
 	{ MCIF_TLSACT,		"TLSACT"	},
 #endif /* STARTTLS */
 	{ MCIF_DLVR_BY,		"DLVR_BY"	},
+	{ MCIF_INLONGLINE,	"INLONGLINE"	},
+	{ MCIF_NOTSTICKY,	"NOTSTICKY"	},
 	{ 0,			NULL		}
 };
 
