@@ -51,16 +51,22 @@
 
 #  ifndef T_TXT
 #   define T_TXT		16
-#  endif /* ! T_TXT */
+#  endif
 #  ifndef T_AFSDB
 #   define T_AFSDB		18
-#  endif /* ! T_AFSDB */
+#  endif
 #  ifndef T_SRV
 #   define T_SRV		33
-#  endif /* ! T_SRV */
+#  endif
 #  ifndef T_NAPTR
 #   define T_NAPTR		35
-#  endif /* ! T_NAPTR */
+#  endif
+# ifndef T_RRSIG
+#  define T_RRSIG		46
+# endif
+# ifndef T_TLSA
+#  define T_TLSA		52
+# endif
 
 typedef struct
 {
@@ -101,10 +107,10 @@ struct resource_record
 		SRV_RECORDT_T	*rr_srv;
 #  if NETINET
 		struct in_addr	*rr_a;
-#  endif /* NETINET */
+#  endif
 #  if NETINET6
 		struct in6_addr *rr_aaaa;
-#  endif /* NETINET6 */
+#  endif
 		char		*rr_txt;
 	} rr_u;
 	RESOURCE_RECORD_T *rr_next;
@@ -113,7 +119,7 @@ struct resource_record
 #  if !defined(T_A) && !defined(T_AAAA)
 /* XXX if <arpa/nameser.h> isn't included */
 typedef int HEADER; /* will never be used */
-#  endif /* !defined(T_A) && !defined(T_AAAA)  */
+#  endif
 
 typedef struct
 {
@@ -122,21 +128,30 @@ typedef struct
 	RESOURCE_RECORD_T	*dns_r_head;
 } DNS_REPLY_T;
 
+#define SM_DNS_FL_EDNS0		0x01
+#define SM_DNS_FL_DNSSEC	0x02
+
+/* flags for parse_dns_reply() */
+#define RR_AS_TEXT	0x01	/* convert some RRs to text, e.g., TLSA */
+#define RR_RAW		0x02	/* return some RRs as "raw" data */
 
 extern void		dns_free_data __P((DNS_REPLY_T *));
 extern int		dns_string_to_type __P((const char *));
 extern const char	*dns_type_to_string __P((int));
-extern DNS_REPLY_T	*dns_lookup_int __P((const char *,
-				int,
-				int,
-				time_t,
-				int));
+extern DNS_REPLY_T	*dns_lookup_map __P((const char *, int, int, time_t,
+				int, unsigned int));
+extern DNS_REPLY_T	*dns_lookup_int __P((const char *, int, int, time_t,
+				int, unsigned int, unsigned int));
 #  if 0
 extern DNS_REPLY_T	*dns_lookup __P((const char *domain,
 				const char *type_name,
 				time_t retrans,
 				int retry));
 #  endif /* 0 */
+#if DNSSEC_TEST
+extern void	dns_setns __P((struct in_addr *, unsigned int));
+extern int	nsportip __P((char *));
+#endif
 
 # endif /* ! __ROKEN_RESOLVE_H__ */
 #endif /* DNSMAP */
