@@ -13,7 +13,7 @@
 
 #include <sendmail.h>
 
-SM_RCSID("@(#)$Id: stab.c,v 8.92 2013/11/22 20:51:56 ca Exp $")
+SM_RCSID("@(#)$Id: stab.c,v 8.92 2013-11-22 20:51:56 ca Exp $")
 
 /*
 **  STAB -- manage the symbol table
@@ -175,6 +175,12 @@ stab(name, type, op)
 		break;
 #endif /* SOCKETMAP */
 
+# if _FFR_TLSA_DANE
+	  case ST_TLSA_RR:
+		len = sizeof(s->s_tlsa);
+		break;
+# endif
+
 	  default:
 		/*
 		**  Each mailer has its own MCI stab entry:
